#include "m_pd.h"
#include <math.h>
#ifdef NT
#pragma warning( disable : 4244 )
#pragma warning( disable : 4305 )
#endif

/* ------------------------ fof~ ----------------------------- */

/* fof~ - first order filters, lowpass and highpass. */

static t_class *fof_class;

typedef struct _fof
{
    t_object x_obj; 	/* obligatory header */
    t_float frequency;  /* place to hold inlet's value if it's set by message */
    t_float frequencyold;
//    t_float coef[10000]; for table lookup
    t_int type;  // low or highpass
    t_float samplerate;
    t_float PI;
    t_float c;
    t_float in1;
    t_float out;
} t_fof;

void fof_lowpass(t_fof *x)
{
    x->type = 0;
}

void fof_highpass(t_fof *x)
{
    x->type = 1;
}

void fof_allpass(t_fof *x)
{
    x->type = 2;
}

void fof_bypass(t_fof *x)
{
    x->type = 3;
}

/* this is the actual performance routine which acts on the samples.
    It's called with a single pointer "w" which is our location in the
    DSP call list.  We return a new "w" which will point to the next item
    after us.  Meanwhile, w[0] is just a pointer to dsp-perform itself
    (no use to us), w[1] and w[2] are the input and output vector locations,
    and w[3] is the number of points to calculate. */
static t_int *fof_perform(t_int *w)
{
    t_float tf;
    t_int coefnum;
    
    t_fof *x = (t_fof *)(w[1]);
    t_float *in = (t_float *)(w[2]);
    t_float *out = (t_float *)(w[3]);
    int n = (int)(w[4]);
    
//    coefnum = (t_int)(x->frequency * 0.5f);
    // calculate coefficient only if frequency changes
    if(x->frequency != x->frequencyold)
    {
       if(x->frequency > (x->samplerate * 0.48f))
            x->frequencyold = x->frequency = (x->samplerate * 0.48f);
        else if(x->frequency < 0.0001f)
            x->frequencyold = x->frequency = 0.0001f;
        else
            x->frequencyold = x->frequency;

        tf = tanf(x->PI * x->frequency/x->samplerate);
        x->c = (tf -1.0f)/(tf + 1.0f);
    }
    
    while (n--)
    {
    	float input = *(in++);
        
        x->out = x->c * (input - x->out) + x->in1;
        x->in1 = input;
        
        switch(x->type)
        {
            case 0:
                *out++ = (input + x->out) * 0.5f;
                break;
            case 1:
                *out++ = (input - x->out) * 0.5f;
                break;
            case 2:
                *out++ = x->out;
                break;
            case 3:
            default:
                *out++ = input;
                break;
        }
    }
    return (w+5);
}

    /* called to start DSP.  Here we call Pd back to add our perform
    routine to a linear callback list which Pd in turn calls to grind
    out the samples. */
static void fof_dsp(t_fof *x, t_signal **sp)
{
    t_int i;
   t_float tf;
    x->samplerate = sp[0]->s_sr;
/*    for(i = 0; i < 10000; i++)
    {
        x->frequency = (2.0f * i);
        if(x->frequency > (x->samplerate * 0.48f))
            x->frequency = (x->samplerate * 0.48f);
        tf = tanf(x->PI * x->frequency/x->samplerate);
        
        x->coef[i] = (tf -1.0f)/(tf + 1.0f);
    }*/
    dsp_add(fof_perform, 4, x, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
}

static void *fof_new(void)
{
    t_int i;
    t_float tf;
    
    t_fof *x = (t_fof *)pd_new(fof_class);
    outlet_new(&x->x_obj, gensym("signal"));
    // initialize state variables

    x->samplerate = 44100.0f;
    x->frequency = x->frequencyold = x->c = 0.0f;
    x->PI = 4.0f * atanf(1.0f);
    x->type = 0;
    x->in1 = x->out = 0.0f;

 /*   for(i = 0; i < 10000; i++)
    {
        x->frequency = (2.0f * i);
        if(x->frequency > (x->samplerate * 0.48f))
            x->frequency = (x->samplerate * 0.48f);
        tf = tanf(x->PI * x->frequency/x->samplerate);
        
        x->coef[i] = (tf -1.0f)/(tf + 1.0f);
    }*/
    return (x);
}

    /* this routine, which must have exactly this name (with the "~" replaced
    by "_tilde) is called when the code is first loaded, and tells Pd how
    to build the "class". */
void fof_tilde_setup(void)
{
    fof_class = class_new(gensym("fof~"), (t_newmethod)fof_new, 0,
    	sizeof(t_fof), 0, A_DEFFLOAT, 0);
	    /* this is magic to declare that the leftmost, "main" inlet
	    takes signals; other signal inlets are done differently... */
    CLASS_MAINSIGNALIN(fof_class, t_fof, frequency);
    class_addmethod(fof_class, (t_method)fof_lowpass, gensym("lowpass"), 0);
    class_addmethod(fof_class, (t_method)fof_highpass, gensym("highpass"), 0);
    class_addmethod(fof_class, (t_method)fof_allpass, gensym("allpass"), 0);
    class_addmethod(fof_class, (t_method)fof_bypass, gensym("bypass"), 0);
    class_addmethod(fof_class, (t_method)fof_dsp, gensym("dsp"), 0);
}
